<?php
/**
 * @file
 * Send patches to testing master server. (Partial code, for drupal.org: 5.x)
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Send all eligible files from last sent timestamp.
 *
 * @param integer $last Last sent timestamp.
 * @param integer $time_interval Time interval in seconds.
 */
function pifr_send_eligible_files($last, $time_interval) {
  $batches = pifr_get_eligible_files($last, $time_interval);

  foreach ($batches as $batch) {
    pifr_send_batch($batch);
  }
}

/**
 * Get eligible files from last sent timestamp, or if last is NULL then get
 * the latest eligible file per node.
 *
 * @param integer $last Last sent timestamp.
 * @param integer $time_interval Time interval in seconds.
 * @return array Set of batches to send.
 */
function pifr_get_eligible_files($last, $time_interval) {
  $batches = array();
  $batch = array();

  // Setup query variables.
  if ($time_interval) {
    $last = ($last == 0 ? time() - $time_interval : $last);
  }
  else {
    $last = 0;
  }
  $pids = variable_get('pifr_project', array());
  $components = explode("\n", variable_get('pifr_component', ''));
  $rids = explode("\n", variable_get('pifr_release', ''));
  $sids = variable_get('pifr_status', array());
  $regex = variable_get('pifr_regex', '\.(patch|diff)$');

  $place_holder_pid = implode("','", array_fill(0, count($pids), '%d'));
  $place_holder_component = implode("','", array_fill(0, count($components), '%s'));
  $place_holder_rid = implode(',', array_fill(0, count($rids), '%d'));
  $place_holder_sid = implode(',', array_fill(0, count($sids), '%d'));

  $query = array_merge(array($last, $regex, $last), $pids, $components, $rids, $sids, array($regex));

  // XXX MySQL only.
  $files = db_query("SELECT n.nid, nf.filepath AS nfilepath, c.cid, cf.filepath AS cfilepath
                     FROM {node} n
                     JOIN {project_issues} ni
                       ON n.nid = ni.nid
                     JOIN {files} nf
                       ON n.nid = nf.nid
                     LEFT JOIN (comments c
                       JOIN {project_issue_comments} ci
                         ON (c.cid = ci.cid AND ci.timestamp >= %d)
                       JOIN {comment_upload_files} cf
                         ON (c.cid = cf.cid AND cf.filepath REGEXP '%s')
                     )
                       ON n.nid = c.nid
                     WHERE n.created >= %d
                     AND ni.pid IN ($place_holder_pid)
                     AND ni.component NOT IN ('$place_holder_component')
                     AND ni.rid IN ($place_holder_rid)
                     AND ni.sid IN ($place_holder_sid)
                     AND nf.filepath REGEXP '%s'
                     ORDER BY n.nid ASC, c.cid DESC, nf.fid", $query);

  if ($time_interval) {
    // Time interval: all files since last run.
    $nid = 0;
    while ($file = db_fetch_array($files)) {
      if (count($batch) >= PIFR_MAX_FILE_COUNT) {
        $batches[] = $batch;
        $batch = array();
      }

      // Always insert comment file.
      if ($file['cid']) {
        $comment = array();
        $comment['nid'] = $file['nid'];
        $comment['cid'] = $file['cid'];
        $comment['filename'] = basename($file['cfilepath']);
        $batch[] = $comment;
      }

      // Only insert node file once.
      if ($file['nid'] != $nid) {
        $node = array();
        $node['nid'] = $file['nid'];
        $node['cid'] = 0;
        $node['filename'] = basename($file['nfilepath']);

        $batch[] = $node;
        $nid = $file['nid'];
      }
    }
  }
  else {
    // No time interval: backlog cron.
    $nid = 0;
    while ($file = db_fetch_array($files)) {
      if (count($batch) >= PIFR_MAX_FILE_COUNT) {
        $batches[] = $batch;
        $batch = array();
      }

      // Only one entry per NID which should be latest due to ORDER BY clause.
      if ($file['nid'] != $nid) {
        // If comment file exists it is the latest so use it.
        $file['filename'] = basename($file['cfilepath'] ? $file['cfilepath'] :$file['nfilepath']);
        unset($file['nfilepath']);
        unset($file['cfilepath']);

        $batch[] = $file;
        $nid = $file['nid'];
      }
    }
  }

  if (count($batch)) {
    $batches[] = $batch;
  }

  return $batches;
}

/**
 * Send batch to master testing server.
 *
 * @param array List of file to send.
 */
function pifr_send_batch($batch) {
  $key = variable_get('pifr_project_key', '');
  $server = variable_get('pifr_test_master_url', '') . 'xmlrpc.php';
  $response = xmlrpc($server, 'pifr.batch.queue', $key, $batch);

  if ($response !== FALSE) {
    if ($response != PIFR_RESPONSE_ACCEPTED) {
      pifr_watchdog_response('Testing master server did not accept file(s)', $response);
    }
  }
  else {
    watchdog('pifr', t('When sending file to testing master server: @message', array('@message' => xmlrpc_error_msg())), WATCHDOG_ERROR);
  }
}

/**
 * Generate watchdog message in the form of: $message: $response.
 *
 * @param string $message Message.
 * @param integer $response_code Response code.
 */
function pifr_watchdog_response($message, $response_code) {
  watchdog('pifr', t("$message: @response",
           array('@response' => pifr_response_code($response_code))),
           WATCHDOG_ERROR);
}

/**
 * Get the response from the code.
 *
 * @param integer $response_code Response code.
 * @return string Representation of reponse code.
 */
function pifr_response_code($response_code) {
  switch ($response_code) {
    case PIFR_RESPONSE_ACCEPTED:
      return t('Accepted');
    case PIFR_RESPONSE_INVALID_SERVER:
      return t('Invalid server');
    case PIFR_RESPONSE_DENIED:
      return t('Denied');
    default:
      return 'Unknown';
  }
}
